{% extends "otree/BaseAdmin.html" %}
{% load otree_tags floppyforms %}

{% block title %}
    Create a new session
{% endblock %}

{% block content %}

{% include "otree/includes/CreateSessionForm.html" %}

{% endblock %}
