{% load staticfiles %}
{% load i18n %}

<!-- you should use .otree-timer if you need to select this element, NOT #otree-timeout -->
<div class="otree-timer alert alert-warning" id="otree-timeout">
    <p>{{ timer_text }}
        <span style="font-weight: bold">
            <span class="glyphicon glyphicon-time"></span>
            <span class="otree-timer__time-left"></span>
        </span>
    </p>
</div>
