{% load defaultlabel from otree_tags %}{% load floppyforms %}{% block row %}
    {% for field in fields %}
        <div class="form-group{% if field.errors %} has-error{% endif %}{% if field.field.required %} required{% endif %}">
            {% with classes=field.css_classes label=label|defaultlabel:field.label help_text=help_text|default:field.help_text %}
                {% block label %}{% if label %}{% if field|id %}
                    <label class="control-label"
                           for="{{ field|id }}">{% endif %}{{ label }}{#{% if field.field.required %} <span class="required">*</span>{% endif %}#}
                {% if label|last not in ".:!?" %}:{% endif %}{% if field|id %}
                    </label>{% endif %}{% endif %}{% endblock %}
                {% block field %}
                    <div class="controls {{ classes }} field-{{ field.name }}">
                        {% block widget %}{% formfield field %}{% endblock %}
                        {% block errors %}
                            {% include "floppyforms/errors.html" with errors=field.errors %}{% endblock %}
                        {% block help_text %}{% if field.help_text %}
                            <p class="help-block">{{ field.help_text }}</p>
                        {% endif %}{% endblock %}
                        {% block hidden_fields %}
                            {% for field in hidden_fields %}
                                {{ field.as_hidden }}{% endfor %}{% endblock %}
                    </div><!--- .controls -->
                {% endblock %}
            {% endwith %}
        </div><!--- .form-group{% if field.errors %}.error{% endif %} -->
    {% endfor %}{% endblock %}
