<!DOCTYPE html>
<html>
<head>
    <title>Server Error (500)</title>
</head>
<body>

<h2>Server Error (500)</h2>

<p>
    There are several ways to find the cause of the issue:
</p>

<ul>
    <li>Set the <code>OTREE_PRODUCTION</code> environment variable back to <code>0</code> and reload this page</li>
    <li>Look at your Sentry messages (see the docs on how to enable Sentry)</li>
    <li>Look at the server logs</li>
</ul>

</body>
</html>